#############################################################################
#
#  This file is part of BGX, the Bayesian Gene eXpression program.
#  Copyright (c) 2003-2004  Graeme Ambler <graeme@ambler.me.uk>
#                2004       Anne-Mette Hein <a.hein@imperial.ac.uk>
#
#  BGX is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License, version 2, as
#  published by the Free Software Foundation.
#
#  BGX is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#############################################################################

fit.bgx<-function(pm,mm,samplesets,probesets,numberGenesToWatch,whichGenesToWatch,whichProbesToWatch,iter=4096,burnin=4096,output="diagnostic",samplenames="CEL files",quantiles=c(0.025,0.5,0.975),init.size=100,subsample=NULL,seed=151789)
  {
    cat("entered bgx.fit function: ")

    #Sort out subsampling
    if(is.null(subsample)) subsample<-as.integer(iter/1024)
    if(subsample<1) subsample<-1
    #make an indicator variable for what type of output we want
    if(output=="minimal") out.ind<-0
    else if(output=="trace") out.ind<-1
    else if(output=="mcse") out.ind<-4
    else if(output=="diagnostic") out.ind<-3
    else if(output=="full") out.ind<-5

    cat("calling .C \n")

    a<-.C("bgx",
          as.double(pm),#pm
          as.double(mm),#mm
          as.integer(dim(pm)[2]),#samples
          as.integer(length(samplesets)),#conditions
          as.integer(dim(pm)[1]),#probes
          as.integer(length(probesets)),#genes
          as.integer(numberGenesToWatch),#number genes to monitor
          as.integer(samplesets),#structure of samples
          as.integer(probesets),#structure of probes
          as.integer(whichGenesToWatch),#gene numbers among analysed to monitor fully
          as.integer(whichProbesToWatch),#first probe numbers for genes to monitor fully
          as.integer(iter),#sweeps
          as.integer(burnin),#burnin
          as.double(150),#s_jump
          as.double(350),#h_jump
          as.double(1.1),#mu_jump
          as.double(1.5),#tau_jump
          as.double(0.04),#lambda_jump
          as.double(0.1),#eta_jump
          as.double(quantiles),#quantiles
          as.integer(length(quantiles)),#num_quantiles
          as.integer(init.size),#init_size
          as.integer(subsample),#subsampling interval
          as.integer(out.ind),#indicator of what output we want
          as.character("gobbledigook"),#returns dirname
          as.integer(seed))#seed
    # writing a file that contains names of CELfiles analysed
    write(samplenames, file = paste(a[[25]],"/summary.txt",sep=""),
          ncolumns = if(is.character(samplenames)) 1 else 5,
          append = TRUE)
    #Just return the name of the output directory
    return(a[[25]])  # was a[[22]]!!!
  }

bgx<-function(data,run="torture",samplesets=NULL,genes=NULL,genesToWatch=NULL,serial=FALSE)
  {
    cat("Entered bgx function\n")
    if(!require(affy))
      stop("The affy package is required and could not be found\n")
    if(run=="torture")
      {
        iter=65536
        burnin=16384
        output="diagnostic"
      }
    else if(run=="experiment")
      {
        iter= 4096
        burnin=1024
        output="diagnostic"
      }
   else if(run=="full")
      {
        iter= 4096
        burnin=1024
        output="full"
      }
    else if(run=="long")
      {
        iter= 262144
        burnin=16384
        output="diagnostic"
      }
    else if(run=="confirm")
      {
        iter=32768
        burnin=8192
        output="trace"
      }
    else if(run=="explore")
      {
        iter=4096
        burnin=4096
        output="minimal"
      }
    else if(run=="short")
      {
        iter=1024
        burnin=1024
        output="diagnostic"
      }
    else
      stop("\"",run,"\" not a recognised option for parameter \"run\"\n")

    if(is.null(genes))
      {
        cat(" parameter 'genes'=NULL: analysing all genes\n")
        genes=geneNames(data)
        geneNumbers=seq(1,length(genes),by=1)
      }
    else
      {
        cat(" parameter 'genes' set: analysing genes: ",genes,"\n")
        geneNumbers=genes
        genes=geneNames(data)[genes]
      }

    pm=pm(data, genes)
    mm=mm(data, genes)
    ip=indexProbes(data,"pm")[genes]
    probesets=rep(0,length(ip))
    for(i in 1:length(ip)) probesets[i]=length(ip[[i]])

    if(is.null(genesToWatch))
      {
        cat(" parameter 'genesToWatch'=NULL: not monitoring any genes in detail.\n")
        numberGenesToWatch = 0
        geneNumbersToWatch = 0
        whichGenesToWatch = 0
        whichProbesToWatch = 0
       }
    else
      {
        cat(" parameter 'genesToWatch' set: monitoring genes: ",genesToWatch,"in detail.\n")
        numberGenesToWatch = length(genesToWatch)
        geneNumbersToWatch=genesToWatch
        genesToWatch=geneNames(data)[genesToWatch]
        whichGenesToWatch=rep(0,length(genesToWatch))
        whichProbesToWatch=rep(0,length(genesToWatch))
        for(g in 1:length(genesToWatch)){
          whichGenesToWatch[g]=match(genesToWatch[g],genes)
          whichProbesToWatch[g]=sum(probesets[1:whichGenesToWatch[g]])-probesets[whichGenesToWatch[g]]+1
        }
      }
    
    numArrays<-ncol(pm(data))
    cat(" Number of arrays to be analysed: ",numArrays,"\n")

    dyn.load("bin/bgx.dll")

    if(serial)
      {
        samplesets=1
        for(c in 1:numArrays)
          {
            outdir=fit.bgx(as.matrix(pm[,c]),as.matrix(mm[,c]),samplesets,probesets,numberGenesToWatch,whichGenesToWatch,whichProbesToWatch,iter,burnin,output,sampleNames(data)[c])      
            cat(" CELfile analysed: ",sampleNames(data)[c]," ... done. Results in: ",outdir,"\n")
          }
        runParameterList=list(data=data,runtype=run,samplesets=samplesets,geneNumbersAnalysed=geneNumbers,geneNumbersWatched=geneNumbersToWatch,serial=TRUE)
        save(runParameterList, file = paste(outdir,"/bgxFunctionCall.Rdata",sep=""))
        return("run finished.")

      }
    else
      {
        if(is.null(samplesets)) samplesets=rep(1,ncol(pm))
        else if(sum(samplesets)!=numArrays) stop("Samplesets not sensible.  You have ", numArrays, " arrays, but sum(samplesets) = ",sum(samplesets))
        outdir=fit.bgx(pm,mm,samplesets,probesets,numberGenesToWatch,whichGenesToWatch,whichProbesToWatch,iter,burnin,output,sampleNames(data))
        cat(" CELfiles analysed: \n")
        for(c in 1:numArrays)
          cat("  ",sampleNames(data)[c],"\n")
        cat(" Results in: ",outdir,"\n")

        namesOfGenesAnalysed=genes

        runParameterList=list(data=data,runtype=run,samplesets=samplesets,geneNumbersAnalysed=geneNumbers,numberProbesAnalysed=sum(probesets),geneNumbersWatched=geneNumbersToWatch,serial=FALSE)
        save(runParameterList, file = paste(outdir,"/bgxFunctionCall.Rdata",sep=""))
        save(namesOfGenesAnalysed, file = paste(outdir,"/namesOfGenesAnalysed.Rdata",sep=""))
        return("run finished.")
      }
  }

# MUST ADD genesToWatch TO THIS!
standalone.bgx<-function(data,run="torture",samplesets=NULL,genes=NULL,prefix="")
  {
    cat("Entered bgx function\n")
    if(!require(affy))
      stop("The affy package is required and could not be found\n")
    if(run=="torture")
      {
        iter=65536
        burnin=16384
        output="diagnostic"
      }
    else if(run=="confirm")
      {
        iter=32768
        burnin=8192
        output="trace"
      }
    else if(run=="short")
      {
        iter=4096
        burnin=4096
        output="diagnostic"
      }
    else if(run=="explore")
      {
        iter=4096
        burnin=4096
        output="minimal"
      }
    else
      stop("\"",run,"\" not a recognised option for parameter \"run\"\n")

    if(is.null(genes))
      {
        cat(" parameter 'genes'=NULL: analysing all genes\n")
        genes=geneNames(data)
      }
    else
      {
        cat(" parameter 'genes' set: analysing genes: ",genes,"\n")
        genes=geneNames(data)[genes]
      }

    pm=pm(data, genes)
    mm=mm(data, genes)
    write(pm,paste(prefix,"PM.txt",sep=""))
    write(mm,paste(prefix,"MM.txt",sep=""))

    ip=indexProbes(data,"pm")[genes]
    probesets=rep(0,length(ip))
    for(i in 1:length(ip)) probesets[i]=length(ip[[i]])
    write(probesets,paste(prefix,"PS.txt",sep=""))

    numArrays<-ncol(pm(data))
    cat(" Number of arrays to be analysed: ",numArrays,"\n")

    if(is.null(samplesets)) samplesets=rep(1,ncol(pm))
    else if(sum(samplesets)!=numArrays) stop("Samplesets not sensible.  You have ", numArrays, " arrays, but sum(samplesets) = ",sum(samplesets))
    write(samplesets,paste(prefix,"SS.txt",sep=""))

    cat(paste("samples",numArrays,"\n"),file=paste(prefix,"infile",sep=""))
    cat(paste("conditions",length(samplesets),"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("probes",dim(pm)[1],"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("genes",length(probesets),"\n"),file=paste(prefix,"infile",sep=""),append=T)

    cat(paste("SampleSets",paste(prefix,"SS.txt",sep=""),"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("ProbeSets",paste(prefix,"PS.txt",sep=""),"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("PM",paste(prefix,"PM.txt",sep=""),"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("MM",paste(prefix,"MM.txt",sep=""),"\n"),file=paste(prefix,"infile",sep=""),append=T)
    
    cat(paste("seed",192492,"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("sweeps",iter,"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat(paste("burn-in",burnin,"\n"),file=paste(prefix,"infile",sep=""),append=T)

    cat(paste("Quantiles 3 0.025 0.5 0.975","\n"),file=paste(prefix,"infile",sep=""),append=T)

    cat(paste("Output",output,"\n"),file=paste(prefix,"infile",sep=""),append=T)
    cat("S_jmp 150\nH_jmp 350\nMu_jmp 1.1\nTau_jmp 1.5\nLambda_jmp 0.04\nEta_jmp 0.1\nInit_size 100\n",file=paste(prefix,"infile",sep=""),append=T)
  }

test.bgx<-function()
  {
    source("data/Sixarraysdata.txt")
    dyn.load("bin/bgx.dll")
    b<-fit.bgx(array.data$PMs,array.data$MMs,c(3,3),c(rep(16,1000),rep(20,11)),1024,1024)
    # Here's an example of how to get the data back into R from the files
    theta<-scan(paste(b,"/thetagave",sep=""))
    theta<-matrix(theta,nc=2)
    plotting(theta,xlab="theta1",ylab="theta2")
    return(b)
  }

plotting<-function(x,...)
  {
    plot(x[1:1000,],...)
    points(x=x[1001,1],y=x[1001,2],pch="2",col="red")
    points(x=x[1002,1],y=x[1002,2],pch="1",col="red")
    points(x=x[1003,1],y=x[1003,2],pch="3",col="red")
    points(x=x[1004,1],y=x[1004,2],pch="9",col="red")
    points(x=x[1005,1],y=x[1005,2],pch="3",col="red")
    points(x=x[1006,1],y=x[1006,2],pch="5",col="red")
    points(x=x[1007,1],y=x[1007,2],pch="7",col="red")
    points(x=x[1008,1],y=x[1008,2],pch="8",col="red")
    points(x=x[1009,1],y=x[1009,2],pch="6",col="red")
    points(x=x[1010,1],y=x[1010,2],pch="A",col="red")
    points(x=x[1011,1],y=x[1011,2],pch="B",col="red")
  }

 testFunctionr<-function(x,y)
  {
    return(list(mor=x,far=y))
  }