/*
 *  This file is part of BGX, the Bayesian Gene eXpression program.
 *  Copyright (c) 2003-2004  Graeme Ambler <graeme@ambler.me.uk>
 *
 *  BGX is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  BGX is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "bgx.hh"
#include <iostream>
#include <fstream>

int main(int argc, const char* argv[])
{
  if(argc<2){
    std::cerr << "No input file specified" << std::endl;
    exit(1);
  }
  std::ifstream infile(argv[1]);
  if(!infile){
    std::cerr << "Cannot open input file" << std::endl;
    exit(2);
  }
  
  int samples=-1;
  int conditions=-1;
  int probes=-1;
  int genes=-1;
  int numberGenesToWatch=-1;
  
  std::string samplesets_file="";
  std::string probesets_file="";
  std::string whichGenesToWatch_file="";  // here
  std::string whichProbesToWatch_file="";  // here
  std::string PM_file="";
  std::string MM_file="";
  
  int seed=-1;
  int iter=-1;
  int burnin=-1;

  double s_jmp=-1;
  double h_jmp=-1;
  double mu_jmp=-1;
  double tau_jmp=-1;
  double lambda_jmp=-1;
  double eta_jmp=-1;
  
  int num_quantiles=-1;
  double* quantiles=NULL;
  
  int init_size=-1;
  int output=0;
  
  std::string temp_str;
  while(infile >> temp_str){
    if(temp_str=="samples"){
      infile >> samples;
    }
    else if(temp_str=="conditions"){
      infile >> conditions;
    }
    else if(temp_str=="probes"){
      infile >> probes;
    }
    else if(temp_str=="genes"){
      infile >> genes;
    }
    else if(temp_str=="SampleSets"){
      infile >> samplesets_file;
    }
    else if(temp_str=="ProbeSets"){
      infile >> probesets_file;
    }
    else if(temp_str=="WhichGenesToWatch"){ // here
      infile >> whichGenesToWatch_file;
    }
    else if(temp_str=="WhichProbesToWatch"){ // here
      infile >> whichProbesToWatch_file;
    }
    else if(temp_str=="PM"){
      infile >> PM_file;
    }
    else if(temp_str=="MM"){
      infile >> MM_file;
    }
    else if(temp_str=="seed"){
      infile >> seed;
    }
    else if(temp_str=="sweeps"){
      infile >> iter;
    }
    else if(temp_str=="burn-in"){
      infile >> burnin;
    }
    else if(temp_str=="S_jmp"){
      infile >> s_jmp;
    }
    else if(temp_str=="H_jmp"){
      infile >> h_jmp;
    }
    else if(temp_str=="Mu_jmp"){
      infile >> mu_jmp;
    }
    else if(temp_str=="Tau_jmp"){
      infile >> tau_jmp;
    }
    else if(temp_str=="Lambda_jmp"){
      infile >> lambda_jmp;
    }
    else if(temp_str=="Eta_jmp"){
      infile >> eta_jmp;
    }
    else if(temp_str=="Quantiles"){
      infile >> num_quantiles;
      quantiles = new double[num_quantiles];
      for(int i=0; i<num_quantiles; ++i)
        infile >> quantiles[i];
    }
    else if(temp_str=="Init_size"){
      infile >> init_size;
    }
    else if(temp_str=="Output"){
      infile >> temp_str;
      if(temp_str=="minimal"){
        output=0;
      }
      else if(temp_str=="trace"){
        output=1;
      }
      else if(temp_str=="mcse"){
        output=4;
      }
      else if(temp_str=="diagnostic"){
        output=3;
      }
      else
        output=0;
    }
    else{
      if(temp_str[0]=='#') getline(infile,temp_str);
      else{
        std::cerr << "Malformed input file at unrecognised token "
		  << temp_str << std::endl;
	exit(3);
      }
    }
  }

  if(samples==-1){
    std::cerr << "Number of samples "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(conditions==-1){
    std::cerr << "Number of conditions "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(probes==-1){
    std::cerr << "Number of probes "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(genes==-1){
    std::cerr << "Number of genes "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(samplesets_file==""){
    std::cerr << "SampleSets file "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(probesets_file==""){
    std::cerr << "ProbeSets file "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(whichGenesToWatch_file==""){       // here
    std::cerr << "whichGenesToWatch file "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(whichProbesToWatch_file==""){       // here
    std::cerr << "whichProbesToWatch file "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(PM_file==""){
    std::cerr << "PM file "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(MM_file==""){
    std::cerr << "MM file "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(seed==-1){
    std::cerr << "Random number seed "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(iter==-1){
    std::cerr << "Number of post burn-in sweeps "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(burnin==-1){
    std::cerr << "Number of burn-in sweeps "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(s_jmp==-1){
    std::cerr << "Spread of jumps for RWM on S "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(h_jmp==-1){
    std::cerr << "Spread of jumps for RWM on H "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(mu_jmp==-1){
    std::cerr << "Spread of jumps for RWM on Mu "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(tau_jmp==-1){
    std::cerr << "Spread of jumps for RWM on Tau "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(lambda_jmp==-1){
    std::cerr << "Spread of jumps for RWM on Lambda "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(eta_jmp==-1){
    std::cerr << "Spread of jumps for RWM on Eta "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  if(num_quantiles==-1){
    std::cerr << "Number and position of Quantiles for Theta "
	      << "not specified in input file" << std::endl;
    exit(11);
  }
  
  if(init_size==-1){
    std::cerr << "Initial sample size for Tierney's method "
	      << "not specified in input file" << std::endl;
    exit(11);
  }

  std::ifstream SampleSets(samplesets_file.c_str());
  if(SampleSets==NULL){
    std::cerr << "Could not open the Sample Sets file" << std::endl;
    exit(4);
  }
  int* samplesets = new int[conditions];
  for(int i=0; i<conditions; ++i)
    SampleSets >> samplesets[i];
  SampleSets.close();

  std::ifstream ProbeSets(probesets_file.c_str());
  if(ProbeSets==NULL){
    std::cerr << "Could not open the Probe Sets file" << std::endl;
    exit(5);
  }
  int* probesets = new int[genes];
  for(int i=0; i<genes; ++i)
    ProbeSets >> probesets[i];
  ProbeSets.close();

std::ifstream WhichGenesToWatch(whichGenesToWatch_file.c_str());  // here
  if(WhichGenesToWatch==NULL){
    std::cerr << "Could not open the Genes To Watch file" << std::endl;
    exit(5);
  }
  int* whichGenesToWatch = new int[numberGenesToWatch];
  for(int i=0; i<numberGenesToWatch; ++i)
    WhichGenesToWatch >> whichGenesToWatch[i];
  WhichGenesToWatch.close();

std::ifstream WhichProbesToWatch(whichProbesToWatch_file.c_str());  // here
  if(WhichProbesToWatch==NULL){
    std::cerr << "Could not open the Probe Sets To Watch file" << std::endl;
    exit(5);
  }
  int* whichProbesToWatch = new int[numberGenesToWatch];
  for(int i=0; i<numberGenesToWatch; ++i)
    WhichProbesToWatch >> whichProbesToWatch[i];
  WhichProbesToWatch.close();
  
  std::ifstream PM(PM_file.c_str());
  if(PM==NULL){
    std::cerr << "Could not open the PM file" << std::endl;
    exit(6);
  }
  std::ifstream MM(MM_file.c_str());
  if(MM==NULL){
    std::cerr << "Could not open the MM file" << std::endl;
    exit(7);
  }
  double* pm = new double[probes*samples];
  double* mm = new double[probes*samples];
  for(int i=0; i<probes*samples; ++i){
    PM >> pm[i];
    MM >> mm[i];
  }
  PM.close();
  MM.close();

  int subsample=iter/1024;
  if(subsample<1) subsample=1;

  char* dirname = new char[100];

  bgx(pm, mm, &samples, &conditions, &probes, &genes, &numberGenesToWatch, samplesets, 
      probesets, whichGenesToWatch, whichProbesToWatch, &iter, &burnin, &s_jmp, &h_jmp, 
      &mu_jmp, &tau_jmp, 
      &lambda_jmp, &eta_jmp, quantiles, &num_quantiles, 
      &init_size, &subsample, &output, &dirname, &seed);

  std::cout << dirname << std::endl;

  delete[] samplesets;
  delete[] probesets;
  delete[] whichGenesToWatch;
  delete[] whichProbesToWatch;
  delete[] pm;
  delete[] mm;
  delete[] dirname;
  return 0;
}
